/**
 * Copyright JS Foundation and other contributors, http://js.foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 **/

var fs = require("fs");
var path = require('path');

var runtime = require("@node-red/runtime");
var redUtil = require("@node-red/util");

var api = require("@node-red/editor-api");

var nodeApp = null;
var adminApp = null;
var server = null;
var apiEnabled = false;

function checkVersion(userSettings) {
    var semver = require('semver');
    if (!semver.satisfies(process.version,">=4.8.0")) {
        // TODO: in the future, make this a hard error.
        // var e = new Error("Unsupported version of node.js");
        // e.code = "unsupported_version";
        // throw e;
        userSettings.UNSUPPORTED_VERSION = process.version;
    }
}

module.exports = {
    init: function(httpServer,userSettings) {
        if (!userSettings) {
            userSettings = httpServer;
            httpServer = null;
        }

        if (!userSettings.SKIP_BUILD_CHECK) {
            checkVersion(userSettings);
        }

        if (!userSettings.coreNodesDir) {
            userSettings.coreNodesDir = path.dirname(require.resolve("@node-red/nodes"))
        }
        redUtil.init(userSettings);
        if (userSettings.httpAdminRoot !== false) {
            runtime.init(userSettings,redUtil,api);
            api.init(httpServer,userSettings,runtime._.storage,runtime);

            apiEnabled = true;
            server = runtime._.adminApi.server;
            runtime.server = runtime._.adminApi.server;
        } else {
            runtime.init(userSettings,redUtil);
            apiEnabled = false;
            if (httpServer){
                server = httpServer;
                runtime._.server = httpServer;
            } else {
                server = runtime._.adminApi.server;
                runtime._.server = runtime._.adminApi.server; // useless at this point, but at least harmless.
            }
        }
        adminApp = runtime._.adminApi.adminApp;
        nodeApp = runtime._.nodeApp;
        return;
    },
    start: function() {
        return runtime.start().then(function() {
            if (apiEnabled) {
                return api.start();
            }
        });
    },
    stop: function() {
        return runtime.stop().then(function() {
            if (apiEnabled) {
                return api.stop();
            }
        })
    },
    nodes: runtime._.nodes,
    log: redUtil.log,
    settings:runtime._.settings,
    util: runtime._.util,
    version: runtime._.version,
    events: runtime._.events,
    comms: {
        publish: function(topic,data,retain) {
            runtime._.events.emit("comms",{
                topic: topic,
                data: data,
                retain: retain
            })
        }
    },
    library: {
        register: function(type) {
            return runtime._.library.register(null,type);
        }
    },
    auth: api.auth,

    get app() { console.log("Deprecated use of RED.app - use RED.httpAdmin instead"); return runtime._.app },
    get httpAdmin() { return adminApp },
    get httpNode() { return nodeApp },
    get server() { return server }
};
